/**
 * Copyright (c) 2005-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.bc;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;

/**
 * Like SimpleServiceUnitManager but start and stop listeners for external addresses.
 * @author Olivier Fabre - EBM WebSourcing
 */
public abstract class BindingComponentServiceUnitManager extends AbstractServiceUnitManager {

    private final ExternalListenerManager externalListenerManager;

    public BindingComponentServiceUnitManager(final AbstractBindingComponent bindingComponent) {
        super(bindingComponent);
        this.externalListenerManager = new ExternalListenerManager(this);
    }

    /**
     * This is used by {@link ExternalListenerManager} to create new external listener for each {@link Consumes} of the
     * SU.
     */
    protected abstract AbstractExternalListener createExternalListener();

    @Override
    protected AbstractBindingComponent getComponent() {
        return (AbstractBindingComponent) super.getComponent();
    }

    @Override
    protected final void deployInternal(final ServiceUnitDataHandler suDH) throws PEtALSCDKException {
        // do nothing
    }

    @Override
    protected final void initInternal(final ServiceUnitDataHandler suDH) throws PEtALSCDKException {
        this.externalListenerManager.createListeners(suDH.getName(), suDH.getDescriptor().getServices().getConsumes());
    }

    @Override
    protected final void startInternal(final ServiceUnitDataHandler suDH) throws PEtALSCDKException {
        this.externalListenerManager.startListening(suDH.getName());
    }

    @Override
    protected final void stopInternal(final ServiceUnitDataHandler suDH) throws PEtALSCDKException {
        // stop the consumption of services for the given SU, as said in the JBI specifications
        this.externalListenerManager.stopListening(suDH.getName());
    }

    @Override
    protected final void shutdownInternal(final ServiceUnitDataHandler suDH) throws PEtALSCDKException {
        this.externalListenerManager.removeListeners(suDH.getName());
    }

    @Override
    protected final void undeployInternal(final ServiceUnitDataHandler suDH) throws PEtALSCDKException {
        // do nothing
    }

}
