/**
 * Copyright (c) 2007-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.listener;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.bc.BindingComponentServiceUnitManager;
import org.ow2.petals.component.framework.bc.ExternalListenerManager;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;

/**
 * The abstract methods for an external listener.
 * 
 * There is one instance of this class per {@link Consumes} (accessible with {@link #getConsumes()}). The class must be
 * declared in the jbi.xml and is instantiated by {@link ExternalListenerManager} (owned by
 * {@link BindingComponentServiceUnitManager}).
 * 
 * It's constructor is invoked when the SU is initialised. The object is destroyed when the SU is shutdown.
 * {@link #start()} is called when the SU starts and {@link #stop()} when the SU stops.
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 */
public abstract class AbstractExternalListener extends AbstractListener {

    /**
     * This method is called after setting of all Listener attributes of {@link AbstractListener} during SU
     * initialisation phase.
     */
    public void init() {
    }

    /**
     * Starts the listener. This method is called when the SU that causes this listener instantiation is started.
     * 
     * @throws PEtALSCDKException
     *             if an error occurs during external listener starting process
     */
    public abstract void start() throws PEtALSCDKException;

    /**
     * Stops the listener. This method is called when the SU that causes this
     * listener instantiation is stopped.
     * 
     * @throws PEtALSCDKException
     *             if an error occurs during external listener stopping process
     */
    public abstract void stop() throws PEtALSCDKException;

    /**
     * Create a message exchange with all the information set in the related
     * {@link Consumes} Su configuration.<br>
     * - service information (itf, srv, ep) <br>
     * - MEP <br>
     * - operation <br>
     * - Properties
     * 
     * @return The message exchange
     * @throws MessagingException
     *             The message exchange can't be created or no pattern specified
     *             in extensions
     * @throws PEtALSCDKException
     *             The current Transaction has failed to return its status
     */
    public Exchange createConsumeExchange() throws MessagingException {
        return this.createConsumeExchange(getConsumes());
    }
}
