/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.mbean;

import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashSet;

import javax.jbi.JBIException;
import javax.jbi.management.MBeanNames;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;

/**
 * A MBean Helper class.
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class MBeanHelper {

	/**
	 * Register a MBean into {@link MBeanServer}
	 * 
	 * @param server
	 * @param objectName
	 * @param object
	 * @throws JBIException
	 */
	public final static void registerMBean(final MBeanServer server,
			final ObjectName objectName, Object object) throws JBIException {
		try {
			if (server == null) {
				throw new JBIException("MBean server is null !");
			}

			if (server.isRegistered(objectName)) {
				server.unregisterMBean(objectName);
			}

			server.registerMBean(object, objectName);

		} catch (final InstanceNotFoundException e) {
			throw new JBIException(e);
		} catch (final MBeanRegistrationException e) {
			throw new JBIException(e);
		} catch (final InstanceAlreadyExistsException e) {
			throw new JBIException(e);
		} catch (final NotCompliantMBeanException e) {
			throw new JBIException(e);
		}
	}

	public final static ObjectName createCustomMBeanName(MBeanNames names, String prefix) {
		return names.createCustomComponentMBeanName(prefix);
	}

	/**
	 * Unregister a MBean from the {@link MBeanServer}
	 * 
	 * @param server
	 * @param objectName
	 * @throws JBIException
	 */
	public final static void unregisterMBean(final MBeanServer server,
			final ObjectName objectName) throws JBIException {
		try {
			server.unregisterMBean(objectName);
		} catch (final InstanceNotFoundException e) {
			throw new JBIException(e);
		} catch (final MBeanRegistrationException e) {
			throw new JBIException(e);
		}
	}

    public static Collection<String> getMethodsNames(final Class<?> clazz) {
        final Collection<String> res = new HashSet<>();
        for (final Method method : clazz.getMethods()) {
            res.add(method.getName());
        }
        return res;
    }
}
