/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.monitoring.defect;

import java.util.Collection;

import org.ow2.petals.component.framework.process.MessageExchangeAcceptor;
import org.ow2.petals.probes.api.sensor.detector.DefectCreator;
import org.ow2.petals.probes.api.sensor.detector.GaugeDefectDetector;

/**
 * Defect detector attached to the message exchange acceptor pool. A defect is
 * created when no more threads is available in the pool.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class MessageExchangeAcceptorPoolDefectDetector implements GaugeDefectDetector<Integer> {

    /**
     * The factory creating the defect about the acceptor pool exhausted.
     */
    private final DefectCreator acceptorPoolExhaustedDefectCreator;

    /**
     * The factory creating the defect about the death of an acceptor.
     */
    private final DefectCreator acceptorPoolAcceptorDeathDefectCreator;

    /**
     * The acceptor pool
     */
    private Collection<MessageExchangeAcceptor> acceptorPool = null;

    /**
     * The expected size of the acceptor pool
     */
    private long acceptorPoolSize = 0;

    /**
     * 
     * @param acceptorPoolExhaustedDefectCreator
     *            The factory creating the defect about the acceptor pool
     *            exhausted.
     * @param acceptorPoolAcceptorDeathDefectCreator
     *            The factory creating the defect about the death of an
     *            acceptor.
     */
    public MessageExchangeAcceptorPoolDefectDetector(
            final DefectCreator acceptorPoolExhaustedDefectCreator,
            final DefectCreator acceptorPoolAcceptorDeathDefectCreator) {
        this.acceptorPoolExhaustedDefectCreator = acceptorPoolExhaustedDefectCreator;
        this.acceptorPoolAcceptorDeathDefectCreator = acceptorPoolAcceptorDeathDefectCreator;
    }

    public final void setAcceptorPool(final Collection<MessageExchangeAcceptor> acceptorPool) {
        this.acceptorPool = acceptorPool;
    }

    public final void setAcceptorPoolSize(final long acceptorPoolSize) {
        this.acceptorPoolSize = acceptorPoolSize;
    }

    @Override
    public void detect(final Integer instantValue) {
        if (this.acceptorPool.isEmpty()) {
            // No more threads is available
            this.acceptorPoolExhaustedDefectCreator.createAndSend();
        } else if (this.acceptorPool.size() != this.acceptorPoolSize) {
            // Few threads are dead
            this.acceptorPoolAcceptorDeathDefectCreator.createAndSend();
        }

    }
}