/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process.async;

import java.util.Iterator;
import java.util.Map;
import java.util.TimerTask;
import java.util.logging.Logger;

import org.ow2.petals.commons.log.FlowAttributes;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.ow2.petals.component.framework.process.JBIProcessorManager;

/**
 * The cleaner task. Remove all the expired exchanges.
 */
public class AsyncContextCleaner extends TimerTask {

    private final Map<String, AsyncContextExchange> asynchronousContexts;

    private final JBIProcessorManager jbiProcessorManager;

    private final Logger logger;

    public AsyncContextCleaner(final Map<String, AsyncContextExchange> asynchronousContexts,
            final JBIProcessorManager jbiProcessorManager, final Logger logger) {
        super();
        this.asynchronousContexts = asynchronousContexts;
        this.jbiProcessorManager = jbiProcessorManager;
        this.logger = logger;
    }

    @Override
    public void run() {
        final Iterator<AsyncContextExchange> it = this.asynchronousContexts.values().iterator();
        while (it.hasNext()) {
            final AsyncContextExchange asyncContextExchange = it.next();
            final AsyncContext asyncContext = asyncContextExchange.getAsyncContext();
            final long expirationTime = asyncContext.getExpirationTime();
            if (expirationTime > 0 && System.currentTimeMillis() > expirationTime) {
                // first let's clean context
                PetalsExecutionContext.clear();

                // A message sent asynchronously has expired
                final ExchangeImpl exchange = asyncContextExchange.getExchange();

                final FlowAttributes current = exchange.getCurrentProcessFlowAttributes();
                // it can't be null because we set it before sending!
                assert current != null;
                PetalsExecutionContext.putFlowAttributes(current);

                this.logger.fine("Expiration of async send for exchange id: " + exchange.getExchangeId());

                // we remove it now!
                it.remove();

                this.jbiProcessorManager.process(exchange, asyncContextExchange);
            }
        }
    }
}
