/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process.async;

import javax.jbi.messaging.MessageExchange;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.message.ExchangeImpl;

/**
 * A wrapper class associating an {@link AsyncContext} with its {@link Exchange}
 * 
 * It contains other informations needed to be remembered without accessing the exchange in case it has expired (because
 * at that point, we don't have the ownership anymore).
 * 
 * @author Christophe DENEUX - EBMWebsourcing
 * 
 */
public class AsyncContextExchange {

    /**
     * The {@link AsyncContext} provided by the component.
     */
    private final AsyncContext asyncContext;

    /**
     * The {@link Exchange} concerned by this context.
     */
    private final ExchangeImpl exchange;

    /**
     * The {@link Consumes} or {@link Provides} concerned by this exchange, if applicable, needed in case of expiration.
     */
    private final Object service;

    /**
     * Needed to remember the role without accessing the {@link MessageExchange} (in case we don't have ownership in
     * expiration).
     */
    private final boolean asProvider;

    public AsyncContextExchange(final AsyncContext asyncContext, final ExchangeImpl exchange, final Object service) {
        this.asyncContext = asyncContext;
        this.exchange = exchange;
        this.service = service;
        this.asProvider = exchange.isProviderRole();
    }

    public AsyncContext getAsyncContext() {
        return asyncContext;
    }

    public ExchangeImpl getExchange() {
        return exchange;
    }

    public Object getService() {
        return service;
    }

    public boolean isAsProvider() {
        return asProvider;
    }
}
