/**
 * Copyright (c) 2005-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;

/**
 * Contains utilities methods for Exchange operations.
 * @author Nicolas Salatge - EBM WebSourcing
 */
public final class ExchangeUtil {
    
    /**
     * Constructor
     */
    public ExchangeUtil() {
        super();
    }

    /**
     * Copies the properties of the exchange in another exchange.
     * 
     * @param source
     *            the source exchange
     * @param target
     *            the target exchange
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    public static final void copyExchangeProperties(final Exchange source, final Exchange target)
            throws MessagingException, PEtALSCDKException {
        if (source.getPropertyNames() != null) {
            for (final String property : source.getPropertyNames()) {
                final String key = (String) property;
                if (key.startsWith(Message.PROPAGATION_PREFIX)) {
                    target.setProperty(key, source.getProperty(key));
                }
            }
        }
    }
}
