/**
 * Copyright (c) 2006-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * Contains utilities methods for XML operations.
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author ddesjardins - EBM WebSourcing
 * @author Gael Blondelle - EBM WebSourcing
 * @TODO use EasyXML XMLUtil class instead, and move all the methods in this project
 */
public final class XMLUtil {

    private XMLUtil() {
        // Do nothing
    }

    /**
     * Return the <i>text</i> element of a node, even if it is contained in a
     * CDATA section
     * 
     * @param node
     * @return
     */
    public static String getTextContent(final Node node) {
        String result = null;
        final NodeList list = node.getChildNodes();
        for (int i = 0; i < list.getLength(); i++) {
            final Node currentNode = list.item(i);
            if (currentNode.getNodeType() == Node.TEXT_NODE
                    || currentNode.getNodeType() == Node.CDATA_SECTION_NODE) {
                result = currentNode.getNodeValue();

                final Node nextSiblingNode = currentNode.getNextSibling();
                if (nextSiblingNode != null
                        && nextSiblingNode.getNodeType() == Node.CDATA_SECTION_NODE) {
                    final String cdataContent = nextSiblingNode.getNodeValue();
                    if (cdataContent != null) {
                        result = result + cdataContent;
                    }
                }

                result = result.replace('\t', ' ').replace('\n', ' ').trim();

                break;
            }
        }
        return result;
    }

    /**
     * Load a document from an input stream.
     * 
     * @param inputStream
     * @return
     * @throws IOException
     * @throws SAXException
     */
    public static Document loadDocument(final InputStream inputStream) throws IOException,
            SAXException {
        Document document = null;

        DocumentBuilder docBuilder = DocumentBuilders.takeDocumentBuilder();
        try {
            document = docBuilder.parse(inputStream);
        } finally {
            DocumentBuilders.releaseDocumentBuilder(docBuilder);
        }
        
        return document;
    }

    /***
     * Recursively traverse node and find the text node with the value of the
     * specified variable.
     * 
     * @param list
     *            The node list to traverse.
     * @param variable
     *            The value of the text node to find.<br/>
     *            (Exemple : ${content})
     * @return The node found otherwise null.
     */
    public static Node findVariable(final NodeList list, final String variable) {
        Node node = null;
        for (int i = 0; i < list.getLength() && node == null; i++) {
            final Node cur = list.item(i);
            if (cur.getNodeType() == Node.TEXT_NODE
                    && cur.getNodeValue().trim().equals(variable)) {
                node = cur;
            } else if (cur.hasChildNodes()) {
                node = findVariable(cur.getChildNodes(), variable);
            }
        }
        return node;
    }
}
