/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.process.async;

import java.lang.ref.PhantomReference;
import java.util.logging.Logger;

import org.junit.Test;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.mbean.RuntimeConfigurationNotifier;
import org.ow2.petals.component.framework.process.AbstractJBIManager;
import org.ow2.petals.component.framework.process.JBIProcessorManager;

import junit.framework.TestCase;

public class AsyncMessageManagerTest extends AbstractJBIManager<AsyncMessageManager> {
    
    private final static Logger LOG = Logger.getLogger(AsyncMessageManagerTest.class.getName());

    private void assertThreadIsPresent() throws Exception {
        super.assertThreadIsPresent(AsyncMessageManager.THREAD_SUFFIX);
    }

    private void assertThreadIsFreed() throws Exception {
        super.assertThreadIsFreed(AsyncMessageManager.THREAD_SUFFIX);
    }

    /**
     * Checks the configuration against a value of time between async cleaner
     * runs set to a valid value. Expected result: the configured value is used.
     * 
     */
    @Test
    public void testGetTimeBeetweenAsyncCleanerRuns_000() throws Exception {

        final long expectedTimeBeetweenAsyncCleanerRuns = 5000l;
        final AbstractComponent componentMock = this.createComponentMock(10, 100, 40, 250, 10, 10, 100, 100,
                expectedTimeBeetweenAsyncCleanerRuns);

        final JBIProcessorManager procMgr = new JBIProcessorManager(componentMock, new RuntimeConfigurationNotifier(
                componentMock, null, null), LOG);

        AsyncMessageManager asyncMsgMgr = new AsyncMessageManager(componentMock, procMgr, LOG);

        asyncMsgMgr.start();

        assertThreadIsPresent();

        // TODO this test is a bit stupid
        TestCase.assertEquals("Time between async cleaner runs is not set to the expected value.",
                expectedTimeBeetweenAsyncCleanerRuns, asyncMsgMgr.getTimeBeetweenAsyncCleanerRuns());

        final PhantomReference<AsyncMessageManager> refAcceptorMgr = new PhantomReference<AsyncMessageManager>(
                asyncMsgMgr, this.queueGC);

        asyncMsgMgr.stop();

        asyncMsgMgr = null;
        assertIsGarbaged(refAcceptorMgr);
        assertThreadIsFreed();
    }
}
