/**
 * Copyright (c) 2006-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.FilenameFilter;
import java.util.LinkedList;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.component.framework.util.FileNamePatternUtil.RegexFileNameFilter;

/**
 * @author Adrien Louis - EBM WebSourcing
 */
public class FileNamePatternUtilTest {

    @Test
    public void testRegexToWildcard() {
        assertEquals("?.XXX?", FileNamePatternUtil.regexToWildcard(".\\.XXX."));
    }

    @Test
    public void testWildcardToRegex() {
        assertEquals(".*\\..*", FileNamePatternUtil.wildcardToRegex("*.*"));
        assertEquals(".\\.XXX.", FileNamePatternUtil.wildcardToRegex("?.XXX?"));
    }

    @Test
    public void testBuildFileNameFilterFromRegex() {
        final RegexFileNameFilter a = FileNamePatternUtil.buildFileNameFilterFromRegex(".*\\.a");
        assertTrue(a.accept(null, "XXX.a"));
        assertTrue(a.accept(null, ".a"));
        assertFalse(a.accept(null, "a"));
        assertFalse(a.accept(null, "XXX.aX"));
    }

    @Test
    public void testBuildFileNameFilterFromWildCard() {
        FilenameFilter a = FileNamePatternUtil.buildFileNameFilterFromWildCard("*.a");
        assertTrue(a.accept(null, "XXX.a"));
        assertTrue(a.accept(null, ".a"));
        assertFalse(a.accept(null, "a"));
        assertFalse(a.accept(null, "XXX.aX"));
        a = FileNamePatternUtil.buildFileNameFilterFromWildCard("X?X.X?X");
        assertTrue(a.accept(null, "XXX.XXX"));
        assertTrue(a.accept(null, "XzX.XzX"));
        assertFalse(a.accept(null, "ZZZ.ZZZ"));
        a = FileNamePatternUtil.buildFileNameFilterFromWildCard("*.*");
        assertTrue(a.accept(null, "X.X"));
        assertFalse(a.accept(null, "XXX"));
    }

    @Test
    public void testBuildFileNameFilterFromFilters() {
        final FilenameFilter a = FileNamePatternUtil.buildFileNameFilterFromWildCard("*.a");
        final FilenameFilter b = FileNamePatternUtil.buildFileNameFilterFromWildCard("*.b");
        final List<FilenameFilter> ab = new LinkedList<>();
        ab.add(a);
        ab.add(b);
        final FilenameFilter abf = FileNamePatternUtil.buildFileNameFilterFromFilters(ab);
        assertTrue(abf.accept(null, "adri.a"));
        assertTrue(abf.accept(null, "adri.b"));
        assertFalse(abf.accept(null, "adri.c"));
    }
}
