/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import static org.junit.Assert.fail;

import java.net.URL;

import javax.xml.namespace.QName;

import org.junit.Before;
import org.junit.Test;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class ServiceUnitUtilTest {

    Description description = null;

    /**
     * @param name
     */
    public ServiceUnitUtilTest() {
        // nothing
    }

    /**
     * {@inheritDoc}
     */
    @Before
    public void setUp() throws Exception {
        final URL url = ServiceUnitUtilTest.class.getResource("/ClockService.wsdl");
        if (url == null) {
            fail("WSDL not found!");
        }
        try {
            this.description = WSDLUtilImpl.createWsdlDescription(url, false);
        } catch (final WSDLException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testAllMatch() throws PEtALSCDKException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock"));

        ServiceUnitUtil.validateDescription(this.description, provides);
    }

    @Test(expected = PEtALSCDKException.class)
    public void testBadInterface() throws PEtALSCDKException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock2"));

        ServiceUnitUtil.validateDescription(this.description, provides);
    }

    @Test(expected = PEtALSCDKException.class)
    public void testBadEndpoint() throws PEtALSCDKException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService2"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock"));

        ServiceUnitUtil.validateDescription(this.description, provides);
    }

    @Test(expected = PEtALSCDKException.class)
    public void testBadService() throws PEtALSCDKException {
        final Provides provides = new Provides();
        provides.setEndpointName("ClockEndpoint2");
        provides.setServiceName(QName.valueOf("{http://petals.ow2.org}ClockService"));
        provides.setInterfaceName(QName.valueOf("{http://petals.ow2.org}Clock"));

        ServiceUnitUtil.validateDescription(this.description, provides);
    }

}
