/**
 * Copyright (c) 2007-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import org.junit.Test;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Christophe Deneux - Cap Gemini
 */
public class SourceUtilTest {

    /**
     * test the createDocument method
     */
    @Test
    public void testCreateDocument() {
        try {
            final Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder()
                    .newDocument();
            final Element element = doc.createElement("test");
            element.setTextContent("coucou");
            doc.appendChild(element);
            final Document finalDocument = SourceUtil.createDocument(new DOMSource(doc));
            assertNotNull(finalDocument);
            assertEquals("coucou", finalDocument.getDocumentElement().getTextContent());
        } catch (final ParserConfigurationException e) {
            fail(e.getMessage());
        } catch (final PEtALSCDKException e) {
            fail(e.getMessage());
        }
    }

    /**
     * test the createDocument method without fork
     */
    @Test
    public void testCreateDocumen2() {
        try {
            final Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder()
                    .newDocument();
            final Element element = doc.createElement("test");
            element.setTextContent("coucou");
            doc.appendChild(element);
            final Document finalDocument = SourceUtil.createDocument(new DOMSource(doc), false);
            assertNotNull(finalDocument);
            assertEquals(doc, finalDocument);
        } catch (final ParserConfigurationException e) {
            fail(e.getMessage());
        } catch (final PEtALSCDKException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testCreateSourceString() {
        final String content = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><result xmlns=\"http://petals.ow2.org/petals-se-eip/scatter-gather\"><EnvoiESB xmlns=\"http://finances.gouv.fr/dgme/esb/webapp/webservice/wsdl/envoiesb\" xmlns:env=\"http://finances.gouv.fr/dgme/esb/webapp/webservice/wsdl/envoiesb\" xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">"
                + "    <informationsGF xmlns=\"\">"
                + "        <idEnveloppe>125</idEnveloppe>"
                + "        <numeroDossier>GFDG</numeroDossier>"
                + "        <destinataire>principal</destinataire>"
                + "        <idEmetteur>GF</idEmetteur>"
                + "    </informationsGF>"
                + "    <informationsSI xmlns=\"\">"
                + "        <idDestinataire>11111111111125</idDestinataire>"
                + "        <idService>3</idService>"
                + "        <idGuichet>1</idGuichet>"
                + "    </informationsSI>"
                + "    <mail xmlns=\"\">"
                + "        <expediteur>component-test@devmail.com</expediteur>"
                + "        <destinataire>component-test@devmail.com</destinataire>"
                + "        <sujet>sujet</sujet>"
                + "        <corps>corps</corps>"
                + "    </mail>"
                + "    <listeFichiers xmlns=\"\">"
                + "        <fichier>"
                + "                <nom>38001-A-8-4E0SNRYB-ent-0.xml</nom>"
                + "        </fichier>"
                + "        <fichier>"
                + "                <nom>38002-A-8-4E0SNRYB-ent-0.xml</nom>"
                + "        </fichier>"
                + "        <fichier>"
                + "                <nom>38003-A-8-4E0SNRYB-ent-0.xml</nom>"
                + "        </fichier>"
                + "    </listeFichiers>" + "</EnvoiESB></result>";
        final Source source = SourceUtil.createSource(content);
        assertNotNull(source);
    }

    @Test
    public void testCreateDocumentSource() {
        final String content = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><result xmlns=\"http://petals.ow2.org/petals-se-eip/scatter-gather\"><EnvoiESB xmlns=\"http://finances.gouv.fr/dgme/esb/webapp/webservice/wsdl/envoiesb\" xmlns:env=\"http://finances.gouv.fr/dgme/esb/webapp/webservice/wsdl/envoiesb\" xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">"
                + "    <informationsGF xmlns=\"\">"
                + "        <idEnveloppe>125</idEnveloppe>"
                + "        <numeroDossier>GFDG</numeroDossier>"
                + "        <destinataire>principal</destinataire>"
                + "        <idEmetteur>GF</idEmetteur>"
                + "    </informationsGF>"
                + "    <informationsSI xmlns=\"\">"
                + "        <idDestinataire>11111111111125</idDestinataire>"
                + "        <idService>3</idService>"
                + "        <idGuichet>1</idGuichet>"
                + "    </informationsSI>"
                + "    <mail xmlns=\"\">"
                + "        <expediteur>component-test@devmail.com</expediteur>"
                + "        <destinataire>component-test@devmail.com</destinataire>"
                + "        <sujet>sujet</sujet>"
                + "        <corps>corps</corps>"
                + "    </mail>"
                + "    <listeFichiers xmlns=\"\">"
                + "        <fichier>"
                + "                <nom>38001-A-8-4E0SNRYB-ent-0.xml</nom>"
                + "        </fichier>"
                + "        <fichier>"
                + "                <nom>38002-A-8-4E0SNRYB-ent-0.xml</nom>"
                + "        </fichier>"
                + "        <fichier>"
                + "                <nom>38003-A-8-4E0SNRYB-ent-0.xml</nom>"
                + "        </fichier>"
                + "    </listeFichiers>" + "</EnvoiESB></result>";
        try {
            final Source source = SourceUtil.createSource(content);
            final Document document = SourceUtil.createDocument(source);
            assertNotNull(document);
        } catch (final PEtALSCDKException e) {
            fail("unable to create the source: " + e.getMessage());
        }
    }
}
