/**
 * Copyright (c) 2010-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class XMLUtilTest {

    @Test
    public void findVariableTrimmed() throws ParserConfigurationException, SAXException, IOException {

        final String baseMsg = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<getResponse>"
                + "<!-- comment after getResponse before base node -->" + "<test>linux rocks</test>"
                + "<base><!-- yet xml comment -->${content}</base>" + "</getResponse>";

        final Node nodeFound = this.findVaraible(baseMsg);

        assertEquals("Check node parent", "base", nodeFound.getParentNode().getLocalName());
    }

    @Test
    public void findVariableNotTrimmed() throws ParserConfigurationException, SAXException, IOException {

        final StringBuilder baseMsg = new StringBuilder();
        baseMsg.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        baseMsg.append('\n');
        baseMsg.append("<getResponse>");
        baseMsg.append('\n');
        baseMsg.append("<!-- comment after getResponse before base node -->");
        baseMsg.append('\n');
        baseMsg.append("<test>linux rocks</test>");
        baseMsg.append('\n');
        baseMsg.append("<base>");
        baseMsg.append('\n');
        baseMsg.append("<!-- yet xml comment -->");
        baseMsg.append('\n');
        baseMsg.append("${content}");
        baseMsg.append('\n');
        baseMsg.append("</base>");
        baseMsg.append('\n');
        baseMsg.append("</getResponse>");

        final Node nodeFound = this.findVaraible(baseMsg.toString());

        assertEquals("Check node parent", "base", nodeFound.getParentNode().getLocalName());
    }

    private Node findVaraible(final String baseMsg) throws ParserConfigurationException, SAXException, IOException {

        final DocumentBuilder builder = DocumentBuilders.takeDocumentBuilder();
        try {
            final Document resultDoc = builder.parse(new ByteArrayInputStream(baseMsg.getBytes()));

            assertNull("Check node not found", XMLUtil.findVariable(resultDoc.getChildNodes(), "${attachments}"));
            final Node nodeFound = XMLUtil.findVariable(resultDoc.getChildNodes(), "${content}");

            assertNotNull("Check node found", nodeFound);
            assertEquals("Check node type", Node.TEXT_NODE, nodeFound.getNodeType());

            return nodeFound;

        } finally {
            DocumentBuilders.releaseDocumentBuilder(builder);
        }
    }
}
